/*
Jonathan Lamb
jonlambusn@gmail.com
*/

#include "CotsBot.h"

CotsBot::CotsBot(bool DB, bool IR){
	val = BT_FLAG;
	motorSpeed = BOTSPEED;		//change this value in .h to speed up or slow down the movements	
	val = BT_FLAG;				//initialize the serial data to a known value
	bump = false;				//clear bump flag
	IRenab = IR;
	DEBUG = DB;
}

CotsBot::CotsBot(){
	val = BT_FLAG;
	motorSpeed = BOTSPEED;		//change this value in .h to speed up or slow down the movements	
	val = BT_FLAG;				//initialize the serial data to a known value
	bump = false;				//clear bump flag
	IRenab = true;				// default to true
	DEBUG = false;				// default to false
}

void CotsBot::init(Encoder *l, Encoder *r){
	Lncod = l;					// point the left encoder object pointer to the right place
	Rncod = r;					// same for right encoder
	pinMode(IRsensor,INPUT);	//set pin 2 as input
	pinMode(M1,OUTPUT);			//set pin 4 as output
	pinMode(E1,OUTPUT);			//set pin 5 as output
	pinMode(E2,OUTPUT);			//set pin 6 as output
	pinMode(M2,OUTPUT);			//set pin 7 as output
	pinMode(switchAnl, INPUT);	// use switch 1, will return a value < 30 when pressed
	//pinMode(audioOut,OUTPUT);	//set pin 10 as output
	pinMode(StatusLed,OUTPUT);	//set pin 11 as output
	pinMode(ErrorLed,OUTPUT);	//set pin 12 as output
	//pinMode(switchS7,INPUT);	//set pin 3 as input
	//pinMode(switchS6,INPUT);	//set pin 2 as input
	pinMode(camLed, OUTPUT);	//set camera led output
	digitalWrite(camLed, HIGH);	// turn led off
	//pinMode(hrdwrLed,OUTPUT);	//set pin 13 as output, on board LED
	//digitalWrite(hrdwrLed, LOW);//turn off on board LED
	digitalWrite(StatusLed,LOW);//turn of the status led
	digitalWrite(ErrorLed,LOW);	//turn of the status led
	//digitalWrite(audioOut,LOW);	//turn the speaker off

	delay(100);		// small start up delay

	if(analogRead(switchAnl)<30)
		digitalWrite(camLed, LOW);

	Lncod->write(encStart);	// set the current position of encoders
	Rncod->write(encStart);

	delay(100);

	regsPulse();

/* if switch 6 is depressed during setup, the front LED will be on, else it will be off */ 
/*
		if(digitalRead(switchS6)==LOW)
		digitalWrite(camLed, LOW);
	else
		digitalWrite(camLed, HIGH);
*/
/*		// if switch 7 is depressed during setup, the bot will poll for an IR trip and hardware LED will be on, else it wont check IR and led off
		if(digitalRead(switchS7)==LOW){
		IRenab = true;
		digitalWrite(hrdwrLed, HIGH);
	}
	else
		IRenab = false;
*/

	//makeSound();				//audio feedback for initialization, uncomment if you have a speaker connected
	//makeSound();	
	Serial.begin(BAUD_RATE);	//setupt the serial connection at the given speed
	delay(500);					//let everything 'settle'

	if(DEBUG==true)
		Serial.println("DEBUGING ENABLED");
}

void CotsBot::forwardEnc(){
	regsPulse();					// register the encoder positions
	long lgoal = lPuls+cmdPls;	// the position we want the encoders to be at
	long rgoal = rPuls+cmdPls;	// same

	// while encoders are not at the desired pos., call forward until they are
	while((lPuls<=lgoal)||(rPuls<=rgoal)){
		forward();
		regsPulse();
	}
	stop();
}

void CotsBot::backwardEnc(){
	regsPulse();					// register the encoder positions
	long lgoal = lPuls-cmdPls;	// the position we want the encoders to be at
	long rgoal = rPuls-cmdPls;	// same

	// while encoders are not at the desired pos., call backward until they are
	while((lPuls>=lgoal)||(rPuls>=rgoal)){
		backward();
		regsPulse();
	}
	stop();
}

void CotsBot::leftEnc(){
	regsPulse();
	long lgoal = lPuls-cmdPls;	// 
	long rgoal = rPuls+cmdPls;	// 

	while((lPuls>=lgoal)||(rPuls<=rgoal)){
		left();
		regsPulse();
	}
	stop();
	
}

void CotsBot::rightEnc(){
	regsPulse();
	long lgoal = lPuls+cmdPls;	// 
	long rgoal = rPuls-cmdPls;	// 

	while((lPuls<=lgoal)||(rPuls>=rgoal)){
		right();
		regsPulse();
	}
	stop();
	
}

void CotsBot::regsPulse(){
	lPuls = (Lncod->read());
	rPuls = (Rncod->read());
}

long CotsBot::getPulse( int enc){
	regsPulse();
	if(enc==0)	// left encoder
		return (lPuls);
	else if(enc==1)	// right encoder
		return (rPuls);
	else
		return 0;
}	// end get pulse

void CotsBot::debugPrint(){
	Serial.println("\n----DEBUG_PRINT----");
	Serial.print("Left: ");
	Serial.println((getPulse(0)), DEC);
	Serial.print("Right: ");
	Serial.println((getPulse(1)), DEC);
	Serial.println("-----------------\n");
}

char CotsBot::getData(){
	if(Serial.available())		//if the serial data is ready set val
		val = Serial.read();
	else						//if it's not available, set to flag var
		val = BT_FLAG;
	return val;
}

void CotsBot::readSerial(){
	if(Serial.available())		//if the serial data is ready set val
		val = Serial.read();
	else						//if it's not available, set to flag
		val = BT_FLAG;
}

/* use this if you want to send data in char's ( ie. E or R) */
void CotsBot::sendData(char data){
	Serial.print(data);	//will send the BYTE of data of the serial connection
}

/* use this if you want to instead send data in raw form, must know how to handle serial data in terms of size
void CotsBot::sendData(dataType data){
	Serial.write(data,DATATYPE);	//will send the BYTE of data of the serial connection
}
*/

/* set the speed of the bot */
void CotsBot::setSpeed(char mSpeed){
	motorSpeed = mSpeed;
}

char CotsBot::getSpeed(){
	return motorSpeed;
}

/*
	The movement functions use polarity shifting and pulse width modulation to control the direction and speed of the two DC motors
*/
void CotsBot::forward(){
	analogWrite(E1,motorSpeed);
	digitalWrite(M1,HIGH);
	analogWrite(E2,motorSpeed);
	digitalWrite(M2,HIGH);
}

void CotsBot::backward(){
	analogWrite(E1,motorSpeed);
	digitalWrite(M1,LOW);
	analogWrite(E2,motorSpeed);
	digitalWrite(M2,LOW);
}

void CotsBot::left(){
	analogWrite(E1,motorSpeed);
	digitalWrite(M1,HIGH);
	analogWrite(E2,motorSpeed);
	digitalWrite(M2,LOW);
}

void CotsBot::right(){
	analogWrite(E1,motorSpeed);
	digitalWrite(M1,LOW);
	analogWrite(E2,motorSpeed);
	digitalWrite(M2,HIGH);
}

void CotsBot::stop(){
	digitalWrite(E1,LOW);
	digitalWrite(E2,LOW);
}

// IR digital sensor on defined PIN, uncomment if this is your setup, in .h too
bool CotsBot::getIR(){
	if(digitalRead(IRsensor)==LOW)
		return true;
	else
		return false;
}

/* This method causes the bot to back up until the sensor no longer detects anything, hardware override to avoid things */
void CotsBot::checkIR(){
	while(digitalRead(IRsensor)==LOW){
		bump = true;
		digitalWrite(ErrorLed,HIGH);
		//analogWrite(audioOut,audioPWM);
		backward();
		delay(PWM_TIME*3);
	}
	stop();
	digitalWrite(ErrorLed,LOW);
	//digitalWrite(audioOut,LOW);
}

/*
void CotsBot::makeSound(){
	analogWrite(audioOut,audioPWM);
	delay(audioLengthMS);
	digitalWrite(audioOut,LOW);
}
*/

/*
This is where all the work is done, Bluetooth mode, add your own fx's here to do other stuff (check sensor's etc)
*/
void CotsBot::recvCmnd(){
	readSerial();				//get the serial BT data
	if(IRenab)
		checkIR();					//check for obstructions if IR check is enabled
	if(val != BT_FLAG){
		digitalWrite(StatusLed,HIGH);	//turn on the status LED when an action is received
		switch(val){
			case FWRD:	//move forward
				forwardEnc();
				break;
			case BKWD:	//move backward
				backwardEnc();
				break;
			case LEFT:	//turn left
				leftEnc();		
				break;
			case RGHT:	//turn right
				rightEnc();
				break;
			case DEBUGC:
				if(DEBUG==true)
					debugPrint();
				break;
			default:
				stop();
				digitalWrite(ErrorLed, HIGH);
				break;
		}// end switch
	}// end if val
	else{
		stop();
		digitalWrite(ErrorLed,HIGH);	//turn on the error led when no action is received
	}
	bump=false;
	digitalWrite(StatusLed,LOW);		//turn off
	digitalWrite(ErrorLed,LOW);			//turn off
	val = BT_FLAG;						//clear the serial data
}

