/*
Jonathan Lamb
jonlambusn@gmail.com

NOTES:
	for the encoder:
		with cmdPls==15, and speed at 0xAD, I achieve 0.5 inch movements on forward and backward with resonable differences
		Start pulse value is 500 for right and left
			call forward -> 
				Left: 532 -> 32 pulse dif.
				Right: 526 -> 26 pulse dif.
			call backward ->
				Left: 468 -> 32 pulse dif.
				Right: 474 -> 26 pulse dif.
		Left and right seem to be pretty close to 0.5 inches as well
*/

#ifndef CotsBot_h
#define CotsBot_h

/*
	// this is already defined in Encoder.h
#if defined(ARDUINO) && ARDUINO >= 100
  #include "Arduino.h"
#else
  #include "WProgram.h"
#endif
*/

#include <../Encoder/Encoder.h>


/*#define CotsBot_VERSION 1		software version	*/
//const bool DEBUG = true;		// debug flag
const int PWM_TIME = 100;		//a delay in ms, sets the delay for each loop which in turn controls how long each action is active
const int BAUD_RATE = 9600;		//Serial data rate, for standard bluetooth modules its 9600
const int M1 = 4;				//Motor 1 direction contro
const int M2 = 7;				//Motor 2 direction control
const int E1 = 5;				//Motor 1 speed control
const int E2 = 6;				//Motor 2 speed control
const int EncRY = 2;			// right motor encoder yellow line
const int EncRW = 11;			// right motor encoder white line
const int EncLY = 3;			// left motor encoder yellow line
const int EncLW = 12;			// left motor encoder white line
const int TX = 1;				// tx hardware pin 1
const int RX = 0;				// rx hardware pin 0
const int IRsensor = 10;		//Infrared sensor signal line
const int camLed = 8;			//camera led
const int switchAnl = A7;		// switches 1-5 use analog pin 7
//const int switchS6 = 2;		//hardware switch 6 (use jumper) on pin 2
//const int switchS7 = 3;		//hardware switch 7 (use jumper) on pin 3
//const int InterruptPin = 0;	//interrupt 0 is on pin 2, interrupt 1 is on pin 3
const int StatusLed = 13;		//green status LED (or any LED) on pin 11
const int ErrorLed = 9;		//red error status LED on pin 12
//const int audioOut = 10;		//speaker/audio output on pin 10
//const int hrdwrLed = 13;		//hardware led on pin 13
const int audioLengthMS = 200;	//length of beep
const int audioPWM = 200;		//PWM value for the speaker
const char FWRD = 0x46;			//forward command, char 'F'
const char BKWD = 0x42;			//backward command, char 'B'
const char LEFT = 0x4C;			//left command, char 'L'
const char RGHT = 0x52;			//right command, char 'R'
const char DEBUGC = 'd';			// debug cammand 
const int BT_FLAG = -1;			//the value of a NO command byte
const int BOTSPEED = 0xAD;		//speed of the bot's motors
const long cmdPls = 15;			// encoder pulse for 0.5 inches for forward/backward
const long encStart = 500;		// value to set both encoders to at startup


class CotsBot{
	public:
		CotsBot();								// default constructor, debug=false, IR=true
		CotsBot(bool, bool);					// custom constructor, 1st arg is for debug, second for check IR
		void init(Encoder *, Encoder *);		//initialize everything, often a lot of this is done is setup()
		char getData();							//if serial data is available then return it, else return -1
		void readSerial();						//read serial data if available, set val value, set to -1 if no data is available
		void sendData(char);					//Send a BYTE of data over the serial connection can be a hex number(0xFF) or a char ('f')
		void setSpeed(char);					//set the speed the motors will use
		char getSpeed();						//get the speed value the motors are using		
		void forward();							//drive the robot forward with a given speed
		void backward();						//drive the robot backward with a given speed
		void left();							//turn the robot left with a given speed
		void right();							//turn the robot right with a given speed
		void stop();							//set both motors to LOW or not moving
		void intrptFnct1();						//function for interrupt on pin 2 (or interrupt0)
		bool getIR();							//check the status of the IR sensor
		void checkIR();							//if if IR sensor is low call a defined fx/Serial command
		void makeSound();						//create a beep with the predefined length and intensity
		void recvCmnd();						//this function will do all the work, receive a serial command and do the action
		long getPulse( int );
		void forwardEnc();
		void backwardEnc();
		void leftEnc();
		void rightEnc();
		void regsPulse();						// register the current encoders position
		void debugPrint();
	private:
		char val;								//the BYTE to store the serial data
		char motorSpeed;						//speed for the motors, MIN:0x00 MAX:0xFF
		bool bump;								//flag set when IR sensor is tripped
		bool IRenab;							// flag for whether or not to check the IR sensor
		Encoder *Lncod;							// left encoder object
		Encoder *Rncod;							// right encoder object
		long lPuls;
		long rPuls;
		bool DEBUG;

};


#endif //CotsBot_h
